/*
    innoIndicator - Toolset to control Delcom USB Visual Indicator devices
    
    SetLED
    v1.1
    
    Copyright (c) 2008 Innotronic Ingenieurbüro Markus Meier
    Copyright (c) 2004 Greg Kroah-Hartmanu <greg@kroah.com>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

 
#include <stdio.h>
#include <string.h>
#include <usb.h>


#define ColorNone		0x00
#define ColorGreen		0x01
#define ColorRed		0x02
#define ColorBlue		0x04
#define ColorYellow		0x03
#define ColorCyan		0x05
#define ColorMagenta	0x06
#define ColorWhite		0x07

#define LED_VENDOR_ID	0x0fc5
#define LED_PRODUCT_ID	0x1223



static struct usb_device *device_init(void)
{
	struct usb_bus *usb_bus;
	struct usb_device *dev;
	
	
	usb_init();
	usb_find_busses();
	usb_find_devices();
	
	for( usb_bus = usb_busses; usb_bus; usb_bus = usb_bus->next )
	{
		for( dev = usb_bus->devices; dev; dev = dev->next )
		{
			if(( dev->descriptor.idVendor == LED_VENDOR_ID) &&
			   ( dev->descriptor.idProduct == LED_PRODUCT_ID))
				return dev;
		}
	}
	
	return NULL;
}


int main(int argc, char **argv)
{
	struct usb_device *usb_dev;
	struct usb_dev_handle *usb_handle;
	
	unsigned char color = ColorNone;
	unsigned char dcOn = 0x80;
	unsigned char dcOff = 0x80;
	
	
	usb_dev = device_init();
	
	if( usb_dev == NULL )
	{
		fprintf( stderr, "Device not found\n") ;
		return( -1 );
	}
	
	
	usb_handle = usb_open( usb_dev );
	
	if( usb_handle == NULL )
	{
		fprintf( stderr, "Not able to claim the USB device\n");
		return( -1 );
	}
	
	
	// Disable all LED's
	usb_control_msg( usb_handle,
					 0x0048,
					 0x0012,
					 0x020a,
					 ColorWhite,
					 NULL,
					 0x0000,
					 1000 );
	
	// Disable FlashMode on all LED's
	usb_control_msg( usb_handle,
					 0x0048,
					 0x0012,
					 0x140a,
					 ColorWhite,
					 NULL,
					 0x0000,
					 1000 );
	
	
	if( argc >= 2 )
	{
		if( strcasecmp( argv[ 1 ], "green" ) == 0 )
			color = ColorGreen;
		
		if( strcasecmp( argv[ 1 ], "red"   ) == 0 )
			color = ColorRed;
		
		if( strcasecmp( argv[ 1 ], "blue"  ) == 0 )
			color = ColorBlue;
		
		if( strcasecmp( argv[ 1 ], "yellow" ) == 0 )
			color = ColorYellow;
		
		if( strcasecmp( argv[ 1 ], "cyan" ) == 0 )
			color = ColorCyan;
		
		if( strcasecmp( argv[ 1 ], "magenta" ) == 0 )
			color = ColorMagenta;
		
		
		if( color & ColorGreen )
		{
			// Set intensity on green
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x220a,
							 0x3200,
							 NULL,
							 0x0000,
							 1000 );
		}
		
		if( color & ColorRed )
		{
			// Set intensity on red
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x220a,
							 0x3201,
							 NULL,
							 0x0000,
							 1000 );
		}
		
		if( color & ColorBlue )
		{
			// Set intensity on blue
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x220a,
							 0x3202,
							 NULL,
							 0x0000,
							 1000 );
		}
		
		
		if( argc >= 3 )
		{
			dcOn = dcOff = atoi( argv[ 2 ] );
			
			if( argc >= 4 )
				dcOff = atoi( argv[ 3 ] );
			
			
			// Set duty cycle on green
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x150a,
							 ( dcOn << 8 ) | dcOff,
							 NULL,
							 0x0000,
							 1000 );
			
			// Set duty cycle on red
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x160a,
							 ( dcOn << 8 ) | dcOff,
							 NULL,
							 0x0000,
							 1000 );
			
			// Set duty cycle on blue
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x170a,
							 ( dcOn << 8 ) | dcOff,
							 NULL,
							 0x0000,
							 1000 );
			
			
			// Enable FlashMode on requested LED's
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x140a,
							 ( color << 8 ),	
							 NULL,
							 0x0000,
							 1000 );
		}
		else
		{
			// Enable requested LED's
			usb_control_msg( usb_handle,
							 0x0048,
							 0x0012,
							 0x020a,
							 0x00ff & ~color,
							 NULL,
							 0x0000,
							 1000 );
		}
	}
	
	usb_close( usb_handle );

	return( 0 );
}
